@@ -6,9 +6,12 @@
 //--------------------------------------------------//
 
 
-FONSEModel::FONSEModel() : Model()
+FONSEModel::FONSEModel(bool _withPhi, bool _fix_sEpsilon) : Model()
 {
 	parameter = 0;
+	withPhi = _withPhi;
+	fix_sEpsilon = _fix_sEpsilon;
+
 }
 
 
@@ -19,7 +22,7 @@ FONSEModel::~FONSEModel()
 }
 
 
-double FONSEModel::calculateLogLikelihoodRatioPerAA(Gene& gene, std::string grouping, double *mutation, double *selection, double phiValue)
+double FONSEModel::calculateLogLikelihoodRatioPerAA(Gene& gene, std::string grouping, double *mutation, double *selection, double phiValue,double a1_value)
 {
 	unsigned numCodons = SequenceSummary::GetNumCodonsForAA(grouping);
 	double logLikelihood = 0.0;
@@ -41,13 +44,12 @@ double FONSEModel::calculateLogLikelihoodRatioPerAA(Gene& gene, std::string grou
 	SequenceSummary::AAToCodonRange(grouping, aaStart, aaEnd, false);
 	for (unsigned i = aaStart, k = 0; i < aaEnd; i++, k++)
 	{
-		positions = gene.geneData.getCodonPositions(i); 
+		positions = gene.geneData.getCodonPositions(i);
 		for (unsigned j = 0; j < positions->size(); j++)
 		{
-			calculateLogCodonProbabilityVector(numCodons, positions->at(j), minIndexVal, mutation, selection, phiValue, codonProb); 
-			if (codonProb[k] == 0) continue;
+			calculateLogCodonProbabilityVector(numCodons, positions->at(j), minIndexVal, mutation, selection, phiValue, a1_value, codonProb);
 			logLikelihood += codonProb[k];
-		} 
+		}
 		//positions->clear();
 	}
  	return logLikelihood;
@@ -76,8 +78,6 @@ double FONSEModel::calculateMutationPrior(std::string grouping, bool proposed)
 
 
 
-
-
 //------------------------------------------------//
 //---------- Likelihood Ratio Functions ----------//
 //------------------------------------------------//
@@ -88,7 +88,6 @@ void FONSEModel::calculateLogLikelihoodRatioPerGene(Gene& gene, unsigned geneInd
 	double likelihood = 0.0;
 	double likelihood_proposed = 0.0;
 	std::string curAA;
-	std::vector <unsigned> positions;
 	double mutation[5];
 	double selection[5];
 
@@ -101,53 +100,53 @@ void FONSEModel::calculateLogLikelihoodRatioPerGene(Gene& gene, unsigned geneInd
 
 	double phiValue = parameter->getSynthesisRate(geneIndex, expressionCategory, false);
 	double phiValue_proposed = parameter->getSynthesisRate(geneIndex, expressionCategory, true);
+	double a1_value = getInitiationCost(false);
 
+	SequenceSummary *sequenceSummary = gene.getSequenceSummary();
 
-	/* TODO: This loop causes a compiler warning because i is an int, but openMP won't compile if I change i to unsigned.
-		Maybe worth looking into? */
 #ifdef _OPENMP
 //#ifndef __APPLE__
-#pragma omp parallel for private(mutation, selection, positions, curAA) reduction(+:likelihood,likelihood_proposed)
+#pragma omp parallel for private(mutation, selection,curAA) reduction(+:likelihood,likelihood_proposed)
 #endif
 	for (unsigned i = 0u; i < getGroupListSize(); i++)
 	{
 		curAA = getGrouping(i);
-
+		if (sequenceSummary->getAACountForAA(i) == 0) continue;
 		parameter->getParameterForCategory(mutationCategory, FONSEParameter::dM, curAA, false, mutation);
 		parameter->getParameterForCategory(selectionCategory, FONSEParameter::dOmega, curAA, false, selection);
-
-		likelihood += calculateLogLikelihoodRatioPerAA(gene, curAA, mutation, selection, phiValue);
-		likelihood_proposed += calculateLogLikelihoodRatioPerAA(gene, curAA, mutation, selection, phiValue_proposed);
+		likelihood += calculateLogLikelihoodRatioPerAA(gene, curAA, mutation, selection, phiValue, a1_value);
+		likelihood_proposed += calculateLogLikelihoodRatioPerAA(gene, curAA, mutation, selection, phiValue_proposed, a1_value);
 	}
 
-	//my_print("% %\n", logLikelihood, logLikelihood_proposed);
-
-	double stdDevSynthesisRate = parameter->getStdDevSynthesisRate(selectionCategory, false);
-	double logPhiProbability = Parameter::densityLogNorm(phiValue, (-(stdDevSynthesisRate * stdDevSynthesisRate) / 2), stdDevSynthesisRate, true);
-	double logPhiProbability_proposed = Parameter::densityLogNorm(phiValue_proposed, (-(stdDevSynthesisRate * stdDevSynthesisRate) / 2), stdDevSynthesisRate, true);
-	double currentLogLikelihood = (likelihood + logPhiProbability);
-	double proposedLogLikelihood = (likelihood_proposed + logPhiProbability_proposed);
-	if (phiValue == 0) {
-		my_print("phiValue is 0\n");
+	unsigned mixture = getMixtureAssignment(geneIndex);
+	mixture = getSynthesisRateCategory(mixture);
+	double stdDevSynthesisRate = parameter->getStdDevSynthesisRate(mixture, false);
+	double mPhi = (-(stdDevSynthesisRate * stdDevSynthesisRate) * 0.5); // X * 0.5 = X / 2
+	//double stdDevSynthesisRate = parameter->getStdDevSynthesisRate(selectionCategory, false);
+	double logPhiProbability = Parameter::densityLogNorm(phiValue, mPhi, stdDevSynthesisRate, true);
+	double logPhiProbability_proposed = Parameter::densityLogNorm(phiValue_proposed, mPhi, stdDevSynthesisRate, true);
+	if (withPhi)
+	{
+		for (unsigned i = 0; i < parameter->getNumObservedPhiSets(); i++)
+		{
+			double obsPhi = gene.getObservedSynthesisRate(i);
+			if (obsPhi > -1.0)
+			{
+				double logObsPhi = std::log(obsPhi);
+				logPhiProbability += Parameter::densityNorm(logObsPhi, std::log(phiValue) + getNoiseOffset(i), getObservedSynthesisNoise(i), true);
+				logPhiProbability_proposed += Parameter::densityNorm(logObsPhi, std::log(phiValue_proposed) + getNoiseOffset(i), getObservedSynthesisNoise(i), true);
+			}
+		}
 	}
-	if (phiValue_proposed == 0) {
-		my_print("phiValue_prop is 0\n");
-	}
-	logProbabilityRatio[0] = (proposedLogLikelihood - currentLogLikelihood) - (std::log(phiValue) - std::log(phiValue_proposed));
-	logProbabilityRatio[1] = currentLogLikelihood - std::log(phiValue_proposed);
-	if (std::isinf(logProbabilityRatio[1])) {
-		my_print("logProb1 inf\n");
-	}
-	logProbabilityRatio[2] = proposedLogLikelihood - std::log(phiValue);
-	if (std::isinf(logProbabilityRatio[2])) {
-		my_print("logProb2 inf\n");
-	}
-
-	//------------NOTE: Jeremy, Cedric changed the reverse jump to where we DON'T include it. I had my PA
-	//LogLikelihood go to 0 because of now missing terms. I have added the code underneath to where we calculate it---/
-
-	logProbabilityRatio[3] = currentLogLikelihood;
-	logProbabilityRatio[4] = proposedLogLikelihood;
+	double currentLogPosterior = (likelihood + logPhiProbability);
+	double proposedLogPosterior = (likelihood_proposed + logPhiProbability_proposed);
+	logProbabilityRatio[0] = (proposedLogPosterior - currentLogPosterior) - (std::log(phiValue) - std::log(phiValue_proposed));
+	logProbabilityRatio[1] = currentLogPosterior - std::log(phiValue_proposed);
+	logProbabilityRatio[2] = proposedLogPosterior - std::log(phiValue);
+	logProbabilityRatio[3] = currentLogPosterior;
+	logProbabilityRatio[4] = proposedLogPosterior;
+	logProbabilityRatio[5] = likelihood;
+	logProbabilityRatio[6] = likelihood_proposed;
 }
 
 
@@ -157,6 +156,7 @@ void FONSEModel::calculateLogLikelihoodRatioPerGroupingPerCategory(std::string g
 	//int numCodons = SequenceSummary::GetNumCodonsForAA(grouping);
 	double likelihood = 0.0;
 	double likelihood_proposed = 0.0;
+	double posterior_proposed,posterior;
 
 	double mutation[5];
 	double selection[5];
@@ -168,10 +168,10 @@ void FONSEModel::calculateLogLikelihoodRatioPerGroupingPerCategory(std::string g
 	Gene *gene;
 	SequenceSummary *sequenceSummary;
 	unsigned aaIndex = SequenceSummary::AAToAAIndex(grouping);
-
+	double a1_value = getInitiationCost(false);
 #ifdef _OPENMP
 //#ifndef __APPLE__
-	#pragma omp parallel for private(mutation, selection, mutation_proposed, selection_proposed, curAA, gene, sequenceSummary) reduction(+:likelihood,likelihood_proposed)
+#pragma omp parallel for private(mutation, selection, mutation_proposed, selection_proposed, curAA, gene, sequenceSummary) reduction(+:likelihood,likelihood_proposed)
 #endif
 	for (unsigned i = 0u; i < numGenes; i++)
 	{
@@ -188,7 +188,6 @@ void FONSEModel::calculateLogLikelihoodRatioPerGroupingPerCategory(std::string g
 		// get phi value, calculate likelihood conditional on phi
 		double phiValue = parameter->getSynthesisRate(i, expressionCategory, false);
 
-
 		// get current mutation and selection parameter
 		parameter->getParameterForCategory(mutationCategory, FONSEParameter::dM, grouping, false, mutation);
 		parameter->getParameterForCategory(selectionCategory, FONSEParameter::dOmega, grouping, false, selection);
@@ -196,20 +195,44 @@ void FONSEModel::calculateLogLikelihoodRatioPerGroupingPerCategory(std::string g
 		// get proposed mutation and selection parameter
 		parameter->getParameterForCategory(mutationCategory, FONSEParameter::dM, grouping, true, mutation_proposed);
 		parameter->getParameterForCategory(selectionCategory, FONSEParameter::dOmega, grouping, true, selection_proposed);
-		
-		likelihood += calculateLogLikelihoodRatioPerAA(*gene, grouping, mutation, selection, phiValue);
-		likelihood_proposed += calculateLogLikelihoodRatioPerAA(*gene, grouping, mutation_proposed, selection_proposed, phiValue);
+		likelihood += calculateLogLikelihoodRatioPerAA(*gene, grouping, mutation, selection, phiValue,a1_value);
+		likelihood_proposed += calculateLogLikelihoodRatioPerAA(*gene, grouping, mutation_proposed, selection_proposed, phiValue,a1_value);
 	}
-	//likelihood_proposed = likelihood_proposed + calculateMutationPrior(grouping, true);
-	//likelihood = likelihood + calculateMutationPrior(grouping, false);
 
-	logAcceptanceRatioForAllMixtures[0] = (likelihood_proposed - likelihood);
+	posterior_proposed = likelihood_proposed + calculateMutationPrior(grouping, true);
+	posterior = likelihood + calculateMutationPrior(grouping, false);
+	logAcceptanceRatioForAllMixtures[0] = (posterior_proposed - posterior);
+	logAcceptanceRatioForAllMixtures[1] = likelihood;
+	logAcceptanceRatioForAllMixtures[2] = likelihood_proposed;
+	logAcceptanceRatioForAllMixtures[3] = posterior;
+	logAcceptanceRatioForAllMixtures[4] = posterior_proposed;
+
 }
 
 
 void FONSEModel::calculateLogLikelihoodRatioForHyperParameters(Genome &genome, unsigned iteration, std::vector <double> & logProbabilityRatio)
 {
+	double mutation[5];
+	double selection[5];
+	std::string curAA;
+	Gene *gene;
+
+	double lpr_sphi = 0.0;
+	double lpr_a1 = 0.0;
 	double lpr = 0.0;
+	if (withPhi)
+	{
+		// one for each noiseOffset, one for stdDevSynthesisRate, one for initiation_cost a1
+		logProbabilityRatio.resize(getNumPhiGroupings() + 2);
+	}
+	else
+	{
+		logProbabilityRatio.resize(2);
+	}
+
+	double a1_current = getInitiationCost(false);
+	double a1_proposed = getInitiationCost(true);
+
 	unsigned selectionCategory = getNumSynthesisRateCategories();
 	std::vector<double> currentStdDevSynthesisRate(selectionCategory, 0.0);
 	std::vector<double> currentMphi(selectionCategory, 0.0);
@@ -218,28 +241,75 @@ void FONSEModel::calculateLogLikelihoodRatioForHyperParameters(Genome &genome, u
 	for (unsigned i = 0u; i < selectionCategory; i++)
 	{
 		currentStdDevSynthesisRate[i] = getStdDevSynthesisRate(i, false);
-		currentMphi[i] = -((currentStdDevSynthesisRate[i] * currentStdDevSynthesisRate[i]) / 2);
+		currentMphi[i] = -((currentStdDevSynthesisRate[i] * currentStdDevSynthesisRate[i]) * 0.5);
 		proposedStdDevSynthesisRate[i] = getStdDevSynthesisRate(i, true);
-		proposedMphi[i] = -((proposedStdDevSynthesisRate[i] * proposedStdDevSynthesisRate[i]) / 2);
+		proposedMphi[i] = -((proposedStdDevSynthesisRate[i] * proposedStdDevSynthesisRate[i]) * 0.5);
 		// take the Jacobian into account for the non-linear transformation from logN to N distribution
-		lpr -= (std::log(currentStdDevSynthesisRate[i]) - std::log(proposedStdDevSynthesisRate[i]));
+		lpr_sphi -= (std::log(currentStdDevSynthesisRate[i]) - std::log(proposedStdDevSynthesisRate[i]));
 	}
 
-	logProbabilityRatio.resize(1);
+	lpr_a1 -= (std::log(a1_current) - std::log(a1_proposed));
 
 #ifdef _OPENMP
 //#ifndef __APPLE__
+#pragma omp parallel for private(gene,mutation, selection, curAA) reduction(+:lpr_sphi,lpr_a1)
+#endif
+	for (unsigned i = 0u; i < genome.getGenomeSize(); i++)
+	{
+		gene = &genome.getGene(i);
+		unsigned mixtureElement = getMixtureAssignment(i);
+		unsigned expressionCategory = getSynthesisRateCategory(mixtureElement);
+		double phi = getSynthesisRate(i, expressionCategory, false);
+		lpr_sphi += Parameter::densityLogNorm(phi, proposedMphi[expressionCategory], proposedStdDevSynthesisRate[expressionCategory], true)
+			   - Parameter::densityLogNorm(phi, currentMphi[expressionCategory], currentStdDevSynthesisRate[expressionCategory], true);
+
+		unsigned mutationCategory = parameter->getMutationCategory(mixtureElement);
+		unsigned selectionCategory = parameter->getSelectionCategory(mixtureElement);
+
+		for (unsigned j = 0u; j < getGroupListSize(); j++)
+		{
+			curAA = getGrouping(j);
+
+			parameter->getParameterForCategory(mutationCategory, FONSEParameter::dM, curAA, false, mutation);
+			parameter->getParameterForCategory(selectionCategory, FONSEParameter::dOmega, curAA, false, selection);
+
+			lpr_a1 += (calculateLogLikelihoodRatioPerAA(*gene, curAA, mutation, selection, phi, a1_proposed) 
+				- calculateLogLikelihoodRatioPerAA(*gene, curAA, mutation, selection, phi, a1_current));
+		}
+	}
+	logProbabilityRatio[0] = lpr_sphi;
+	logProbabilityRatio[1] = lpr_a1;
+	if (withPhi)
+	{
+		for (unsigned i = 0; i < parameter->getNumObservedPhiSets(); i++)
+		{
+			
+			lpr = 0.0;
+			double noiseOffset = getNoiseOffset(i, false);
+			double noiseOffset_proposed = getNoiseOffset(i, true);
+			double observedSynthesisNoise = getObservedSynthesisNoise(i);
+#ifdef _OPENMP
+//#ifndef __APPLE__
 #pragma omp parallel for reduction(+:lpr)
 #endif
-	for (unsigned i = 0u; i < genome.getGenomeSize(); i++)
-	{
-		unsigned mixture = getMixtureAssignment(i);
-		mixture = getSynthesisRateCategory(mixture);
-		double phi = getSynthesisRate(i, mixture, false);
-		lpr += Parameter::densityLogNorm(phi, proposedMphi[mixture], proposedStdDevSynthesisRate[mixture], true)
-			   - Parameter::densityLogNorm(phi, currentMphi[mixture], currentStdDevSynthesisRate[mixture], true);
+			for (unsigned j = 0u; j < genome.getGenomeSize(); j++)
+			{
+				unsigned mixtureAssignment = getMixtureAssignment(j);
+				mixtureAssignment = getSynthesisRateCategory(mixtureAssignment);
+				double logPhi = std::log(getSynthesisRate(j, mixtureAssignment, false));
+				double obsPhi = genome.getGene(j).getObservedSynthesisRate(i);
+				if (obsPhi > -1.0)
+				{
+					double logObsPhi = std::log(obsPhi);
+					double proposed = Parameter::densityNorm(logObsPhi, logPhi + noiseOffset_proposed, observedSynthesisNoise, true);
+					double current = Parameter::densityNorm(logObsPhi, logPhi + noiseOffset, observedSynthesisNoise, true);
+					lpr += proposed - current;
+				}
+			}
+			logProbabilityRatio[i+2] = lpr;
+		}
 	}
-	logProbabilityRatio[0] = lpr;
+
 }
 
 
@@ -251,9 +321,9 @@ void FONSEModel::calculateLogLikelihoodRatioForHyperParameters(Genome &genome, u
 //----------------------------------------------------------//
 
 
-void FONSEModel::initTraces(unsigned samples, unsigned num_genes)
+void FONSEModel::initTraces(unsigned samples, unsigned num_genes, bool estimateSynthesisRate)
 {
-	parameter->initAllTraces(samples, num_genes);
+	parameter->initAllTraces(samples, num_genes, estimateSynthesisRate);
 }
 
 
@@ -301,6 +371,10 @@ std::vector<unsigned> FONSEModel::getMixtureElementsOfSelectionCategory(unsigned
 }
 
 
+double FONSEModel::getInitiationCost(bool proposed)
+{
+	return parameter->getInitiationCost(proposed);
+}
 
 
 
@@ -347,6 +421,17 @@ void FONSEModel::updateStdDevSynthesisRate()
 }
 
 
+double FONSEModel::getCurrentInitiationCostProposalWidth()
+{
+	return parameter->getCurrentInitiationCostProposalWidth();
+}
+
+void FONSEModel::updateInitiationCost()
+{
+	parameter->updateInitiationCost();
+}
+
+
 
 
 
@@ -400,6 +485,10 @@ void FONSEModel::updateStdDevSynthesisRateTrace(unsigned sample)
 	parameter->updateStdDevSynthesisRateTrace(sample);
 }
 
+void FONSEModel::updateInitiationCostParameterTrace(unsigned sample)
+{
+	parameter->updateInitiationCostParameterTrace(sample);
+}
 
 void FONSEModel::updateSynthesisRateTrace(unsigned sample, unsigned i)
 {
@@ -428,6 +517,12 @@ void FONSEModel::updateCodonSpecificParameterTrace(unsigned sample, std::string
 void FONSEModel::updateHyperParameterTraces(unsigned sample)
 {
 	updateStdDevSynthesisRateTrace(sample);
+	updateInitiationCostParameterTrace(sample);
+	if (withPhi)
+	{
+		updateNoiseOffsetTrace(sample);
+    	updateObservedSynthesisNoiseTrace(sample);
+    }
 }
 
 
@@ -462,6 +557,11 @@ void FONSEModel::adaptStdDevSynthesisRateProposalWidth(unsigned adaptiveWidth, b
 }
 
 
+void FONSEModel::adaptInitiationCostProposalWidth(unsigned adaptiveWidth, bool adapt)
+{
+	parameter->adaptInitiationCostProposalWidth(adaptiveWidth, adapt);
+}
+
 void FONSEModel::adaptSynthesisRateProposalWidth(unsigned adaptiveWidth, bool adapt)
 {
 	parameter->adaptSynthesisRateProposalWidth(adaptiveWidth, adapt);
@@ -477,6 +577,9 @@ void FONSEModel::adaptCodonSpecificParameterProposalWidth(unsigned adaptiveWidth
 void FONSEModel::adaptHyperParameterProposalWidths(unsigned adaptiveWidth, bool adapt)
 {
 	adaptStdDevSynthesisRateProposalWidth(adaptiveWidth, adapt);
+	adaptInitiationCostProposalWidth(adaptiveWidth,adapt);
+	if (withPhi)
+		adaptNoiseOffsetProposalWidth(adaptiveWidth, adapt);
 }
 
 
@@ -497,6 +600,10 @@ void FONSEModel::proposeCodonSpecificParameter()
 void FONSEModel::proposeHyperParameters()
 {
 	parameter->proposeHyperParameters();
+	if (withPhi)
+	{
+		parameter->proposeNoiseOffset();
+	}
 }
 
 
@@ -553,29 +660,127 @@ void FONSEModel::updateCodonSpecificParameter(std::string grouping)
 	parameter->updateCodonSpecificParameter(grouping);
 }
 
+void FONSEModel::completeUpdateCodonSpecificParameter()
+{
+    parameter->completeUpdateCodonSpecificParameter();
+}
+
+//Noise offset functions
+
+double FONSEModel::getNoiseOffset(unsigned index, bool proposed)
+{
+	return parameter->getNoiseOffset(index, proposed);
+}
+
+
+double FONSEModel::getObservedSynthesisNoise(unsigned index)
+{
+	return parameter->getObservedSynthesisNoise(index);
+}
+
+
+double FONSEModel::getCurrentNoiseOffsetProposalWidth(unsigned index)
+{
+	return parameter->getCurrentNoiseOffsetProposalWidth(index);
+}
+
+
+void FONSEModel::updateNoiseOffset(unsigned index)
+{
+	parameter->updateNoiseOffset(index);
+}
+
+
+void FONSEModel::updateNoiseOffsetTrace(unsigned sample)
+{
+	parameter->updateNoiseOffsetTraces(sample);
+}
+
+
+void FONSEModel::updateObservedSynthesisNoiseTrace(unsigned sample)
+{
+	parameter->updateObservedSynthesisNoiseTraces(sample);
+}
+
+
+void FONSEModel::adaptNoiseOffsetProposalWidth(unsigned adaptiveWidth, bool adapt)
+{
+	parameter->adaptNoiseOffsetProposalWidth(adaptiveWidth, adapt);
+}
+
+
 
 void FONSEModel::updateGibbsSampledHyperParameters(Genome &genome)
 {
-	//TODO: fill in
+  // estimate s_epsilon by sampling from a gamma distribution and transforming it into an inverse gamma sample
+	
+	if (withPhi)
+	{
+		if(!fix_sEpsilon)
+		{
+			double shape = ((double)genome.getGenomeSize() - 1.0) / 2.0;
+			for (unsigned i = 0; i < parameter->getNumObservedPhiSets(); i++)
+			{
+				double rate = 0.0; //Prior on s_epsilon goes here?
+				unsigned mixtureAssignment;
+				double noiseOffset = getNoiseOffset(i);
+				for (unsigned j = 0; j < genome.getGenomeSize(); j++)
+				{
+					mixtureAssignment = parameter->getMixtureAssignment(j);
+					double obsPhi = genome.getGene(j).getObservedSynthesisRate(i);
+					if (obsPhi > -1.0)
+					{
+						double sum = std::log(obsPhi) - noiseOffset - std::log(parameter->getSynthesisRate(j, mixtureAssignment, false));
+						rate += (sum * sum);
+					}else{
+						// missing observation.
+						shape -= 0.5;
+						//Reduce shape because initial estimate assumes there are no missing observations
+					}
+				}
+				rate /= 2.0;
+				double rand = parameter->randGamma(shape, rate);
+
+				// Below the gamma sample is transformed into an inverse gamma sample
+				// According to Gilchrist et al (2015) Supporting Materials p. S6
+				// The sample 1/T is supposed to be equal to $s_\epsilon^2$.
+				double sepsilon = std::sqrt(1.0/rand);
+				parameter->setObservedSynthesisNoise(i, sepsilon);
+			}
+		}
+	}
 }
 
 
+
 void FONSEModel::updateAllHyperParameter()
 {
 	updateStdDevSynthesisRate();
+	updateInitiationCost();
+	if (withPhi)
+	{
+		for (unsigned i =0; i < parameter->getNumObservedPhiSets(); i++)
+		{
+			updateNoiseOffset(i);
+		}
+	}
 }
 
 void FONSEModel::updateHyperParameter(unsigned hp)
 {
 	// NOTE: when adding additional hyper parameter, also add to updateAllHyperParameter()
-	switch (hp)
+	if (hp == 0)
 	{
-		case 0:
-			updateStdDevSynthesisRate();
-			break;
-		default:
-			updateStdDevSynthesisRate();
-			break;
+		updateStdDevSynthesisRate();
+	}
+	else if (hp == 1)
+	{
+		updateInitiationCost();
+	}		
+	else if (hp > 1 and withPhi)
+	{	
+		//subtract off 2 because the first two parameters withh be the updateStdDevSynthesisRate
+		updateNoiseOffset(hp - 2);
 	}
 }
 
@@ -589,18 +794,15 @@ void FONSEModel::simulateGenome(Genome & genome)
 
 	for (unsigned geneIndex = 0; geneIndex < genome.getGenomeSize(); geneIndex++) //loop over all genes in the genome
 	{
-		if (geneIndex % 100 == 0) my_print("Simulating Gene %\n", geneIndex);
 		Gene gene = genome.getGene(geneIndex);
 		SequenceSummary sequenceSummary = gene.geneData;
 		std::string tmpSeq = "ATG"; //Always will have the start amino acid
-
-
 		unsigned mixtureElement = getMixtureAssignment(geneIndex);
 		unsigned mutationCategory = getMutationCategory(mixtureElement);
 		unsigned selectionCategory = getSelectionCategory(mixtureElement);
 		unsigned synthesisRateCategory = getSynthesisRateCategory(mixtureElement);
 		double phi = getSynthesisRate(geneIndex, synthesisRateCategory, false);
-
+		double a1_value = getInitiationCost(false);
 		std::string geneSeq = gene.getSequence();
 		for (unsigned position = 1; position < (geneSeq.size() / 3); position++)
 		{
@@ -628,7 +830,7 @@ void FONSEModel::simulateGenome(Genome & genome)
 			{
 				getParameterForCategory(mutationCategory, FONSEParameter::dM, curAA, false, mutation);
 				getParameterForCategory(selectionCategory, FONSEParameter::dOmega, curAA, false, selection);
-				calculateCodonProbabilityVector(numCodons, position, mutation, selection, phi, codonProb);
+				calculateCodonProbabilityVector(numCodons, position, mutation, selection, phi, a1_value,codonProb);
 			}
 
 
@@ -653,6 +855,9 @@ void FONSEModel::printHyperParameters()
 		my_print("stdDevSynthesisRate posterior estimate for selection category %: %\n", i, getStdDevSynthesisRate(i));
 	}
 	my_print("\t current stdDevSynthesisRate proposal width: %\n", getCurrentStdDevSynthesisRateProposalWidth());
+	my_print("\t current initiation cost a_1 estimate: %\n",getInitiationCost(false));
+	my_print("\t current initiation cost a_1 proposal width: %\n", getCurrentInitiationCostProposalWidth());
+
 }
 
 
@@ -690,7 +895,7 @@ double FONSEModel::calculateAllPriors()
 
 //Calculates the log probability of each codon for an amino acid and puts them in a vector.
 void FONSEModel::calculateLogCodonProbabilityVector(unsigned numCodons, unsigned position, unsigned minIndexValue,
-												 double *mutation, double *selection, double phi, std::vector <double> &codonProb)
+												 double *mutation, double *selection, double phi, double a1_value, std::vector <double> &codonProb)
 {
 	double denominator;
 
@@ -705,15 +910,16 @@ void FONSEModel::calculateLogCodonProbabilityVector(unsigned numCodons, unsigned
 	 // If the reference codon is the min value (0) then we do not have to adjust the reference codon.
 	 // This is necessary to deal with very large phi values (> 10^4) and avoid producing Inf which then
 	 // causes the denominator to be Inf (Inf / Inf = NaN).
-	if (selection[minIndexValue] < 0.0) {
+	if (selection[minIndexValue] < 0.0)
+	{
 		denominator = 0.0;
 		for (unsigned i = 0u; i < (numCodons - 1); i++)
 		{
-			codonProb[i] = -(mutation[i] - mutation[minIndexValue]) - (phi * (4.0 + (4.0 * position)) * (selection[i] - selection[minIndexValue]));
+			codonProb[i] = -(mutation[i] - mutation[minIndexValue]) - (phi * (a1_value + (4.0 * position)) * (selection[i] - selection[minIndexValue]));
 			denominator += std::exp(codonProb[i]);
 		}
 		//Alphabetically, the last codon is the reference codon.
-		codonProb[numCodons - 1] = (mutation[minIndexValue]) + (phi * (4.0 + (4.0 * position)) * selection[minIndexValue]);
+		codonProb[numCodons - 1] = (mutation[minIndexValue]) + (phi * (a1_value + (4.0 * position)) * selection[minIndexValue]);
 		denominator += std::exp(codonProb[numCodons - 1]);
 	}
 	else
@@ -721,14 +927,14 @@ void FONSEModel::calculateLogCodonProbabilityVector(unsigned numCodons, unsigned
 		denominator = 1.0;
 		for (unsigned i = 0u; i < (numCodons - 1); i++)
 		{
-			codonProb[i] = -(mutation[i]) - (phi * (4.0 + (4.0 * position)) * selection[i]);
+			codonProb[i] = -(mutation[i]) - (phi * (a1_value + (4.0 * position)) * selection[i]);
 			denominator += std::exp(codonProb[i]);
 		}
 		//Again, the last codon is the reference codon
 		codonProb[numCodons - 1] = 0.0;
 	}
 
-	//Here we take the log of the denominator (the summation term) so that we can finish calculating 
+	//Here we take the log of the denominator (the summation term) so that we can finish calculating
 	//the log probabilities simple by subtracting the log of the denominator from each element.
 	denominator = std::log(denominator);
 	for (unsigned i = 0; i < numCodons; i++)
@@ -741,7 +947,7 @@ void FONSEModel::calculateLogCodonProbabilityVector(unsigned numCodons, unsigned
 //Since the simulateGenome function utilizes the codon probability vector, but doesn't deal with the log values,
 //this function simply returns the vector with each codon's probability.
 void FONSEModel::calculateCodonProbabilityVector(unsigned numCodons, unsigned position,
-													double *mutation, double *selection, double phi, double codonProb[])
+													double *mutation, double *selection, double phi, double a1_value, double codonProb[])
 {
 	double denominator;
 	unsigned minIndexValue = 0u;
@@ -758,7 +964,9 @@ void FONSEModel::calculateCodonProbabilityVector(unsigned numCodons, unsigned po
 	*                                                                          *
 	* Right now a_1 and a_2 are set to 4.0. However, we are planning on making *
 	* them hyperparameters in the future, since they are constant for the      *
-	* entire genome.                                                           */
+	* entire genome.
+	* Note codon position, indexing starts at 0,thus we don't need (i-1)
+	* \Delta \omega includes q Ne terms                                        */
 
 
 	// If the min(selection) is less than zero than we have to adjust the reference codon.
@@ -769,11 +977,11 @@ void FONSEModel::calculateCodonProbabilityVector(unsigned numCodons, unsigned po
 		denominator = 0.0;
 		for (unsigned i = 0u; i < (numCodons - 1); i++)
 		{
-			codonProb[i] = std::exp(-(mutation[i] - mutation[minIndexValue]) - (phi * (4.0 + (4.0 * position)) * (selection[i] - selection[minIndexValue])));
+			codonProb[i] = std::exp(-(mutation[i] - mutation[minIndexValue]) - (phi * (a1_value + (4.0 * position)) * (selection[i] - selection[minIndexValue])));
 			denominator += codonProb[i];
 		}
 		//Alphabetically, the last codon is the reference codon.
-		codonProb[numCodons - 1] = std::exp((mutation[minIndexValue]) + (phi * (4.0 + (4.0 * position)) * selection[minIndexValue]));
+		codonProb[numCodons - 1] = std::exp((mutation[minIndexValue]) + (phi * (a1_value + (4.0 * position)) * selection[minIndexValue]));
 		denominator += codonProb[numCodons - 1];
 	}
 	else
@@ -781,14 +989,14 @@ void FONSEModel::calculateCodonProbabilityVector(unsigned numCodons, unsigned po
 		denominator = 1.0;
 		for (unsigned i = 0u; i < (numCodons - 1); i++)
 		{
-			codonProb[i] = std::exp(-(mutation[i]) - (phi * (4.0 + (4.0 * position)) * selection[i]));
+			codonProb[i] = std::exp(-(mutation[i]) - (phi * (a1_value + (4.0 * position)) * selection[i]));
 			denominator += codonProb[i];
 		}
 		//Again, the last codon is the reference codon
 		codonProb[numCodons - 1] = 1.0;
 	}
 
-	//As is found in ROCModel.cpp, multiplication is a faster operation than division so we 
+	//As is found in ROCModel.cpp, multiplication is a faster operation than division so we
 	//save time here by dividing once and then multiplying numCodons times instead of dividing
 	//numCodons times.
 	denominator = 1 / denominator;
@@ -798,6 +1006,8 @@ void FONSEModel::calculateCodonProbabilityVector(unsigned numCodons, unsigned po
 	}
 }
 
+
+
 void FONSEModel::getParameterForCategory(unsigned category, unsigned param, std::string aa, bool proposal, double* returnValue)
 {
 	parameter->getParameterForCategory(category, param, aa, proposal, returnValue);
@@ -818,16 +1028,16 @@ void FONSEModel::getParameterForCategory(unsigned category, unsigned param, std:
 #ifndef STANDALONE
 
 
-/*std::vector<double> FONSEModel::CalculateProbabilitiesForCodons(std::vector<double> mutation, std::vector<double> selection, double phi)
+std::vector<double> FONSEModel::CalculateProbabilitiesForCodons(std::vector<double> mutation, std::vector<double> selection, double phi, double a1_value, unsigned position)
 {
 	unsigned numCodons = mutation.size() + 1;
 	double* _mutation = &mutation[0];
 	double* _selection = &selection[0];
 	double* codonProb = new double[numCodons]();
-	calculateCodonProbabilityVector(numCodons, _mutation, _selection, phi, codonProb);
+	calculateCodonProbabilityVector(numCodons, position, _mutation, _selection, phi, a1_value, codonProb);
 	std::vector<double> returnVector(codonProb, codonProb + numCodons);
 	return returnVector;
-}*/
+}
 
 
 #endif
