@@ -14,7 +14,9 @@ ROCParameter::ROCParameter() : Parameter()
 {
 	//CTOR
 	bias_csp = 0;
-	mutation_prior_sd = 0.35;
+	//mutation_prior_sd = 0.35;
+	mutation_prior_mean.resize(40);
+	mutation_prior_sd.resize(40);
 	currentCodonSpecificParameter.resize(2);
 	proposedCodonSpecificParameter.resize(2);
 }
@@ -74,21 +76,32 @@ ROCParameter::~ROCParameter()
 
 void ROCParameter::initROCParameterSet()
 {
-	mutation_prior_sd = 0.35;
-
+	//mutation_prior_sd = 0.35;
+	mutation_prior_mean.resize(numMutationCategories);
+	mutation_prior_sd.resize(numMutationCategories);
+	for (int i=0; i < numMutationCategories; i++)
+	{
+	  // POTENTIAL ISSUE: Shouldn't we use (numParam) instead of (40)
+		mutation_prior_mean[i].resize(40);
+		mutation_prior_sd[i].resize(40);
+		std::vector<double> tmp(40, 0.0);
+		mutation_prior_mean[i] = tmp;
+		mutation_prior_sd[i] = tmp;
+	}
+	// POTENTIAL ISSUE: This seems to assume splitSer = True
 	groupList = {"A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "N", "P", "Q", "R", "S", "T", "V", "Y", "Z"};
 	// proposal bias and std for codon specific parameter
 	bias_csp = 0;
-	
 
-	for (unsigned i = 0; i < getNumObservedPhiSets(); i++)
-	{
-		noiseOffset[i] = 0.1;
-		noiseOffset_proposed[i] = 0.1;
-		std_NoiseOffset[i] = 0.1;
-		observedSynthesisNoise[i] = 0.1;
-		numAcceptForNoiseOffset[i] = 0;
-	}
+
+	// for (unsigned i = 0; i < getNumObservedPhiSets(); i++)
+	// {
+	// 	noiseOffset[i] = 0.1;
+	// 	noiseOffset_proposed[i] = 0.1;
+	// 	std_NoiseOffset[i] = 0.1;
+	// 	observedSynthesisNoise[i] = 0.1;
+	// 	numAcceptForNoiseOffset[i] = 0;
+	// }
 
 	//may need getter fcts
 
@@ -107,7 +120,6 @@ void ROCParameter::initROCParameterSet()
 
 	currentCodonSpecificParameter[dEta].resize(numSelectionCategories);
 	proposedCodonSpecificParameter[dEta].resize(numSelectionCategories);
-
 	for (unsigned i = 0u; i < numSelectionCategories; i++)
 	{
 		std::vector<double> tmp(numParam, 0.0);
@@ -128,14 +140,19 @@ void ROCParameter::initROCParameterSet()
 
 void ROCParameter::initROCValuesFromFile(std::string filename)
 {
+	bool old_format = true;
 	std::ifstream input;
 	covarianceMatrix.resize(maxGrouping);
 	std::vector <double> mat;
 	input.open(filename.c_str());
 	if (input.fail())
-		my_printError("Error opening file % to initialize from restart file.\n", filename.c_str());
-	else
 	{
+		my_printError("Error opening file % to initialize from restart file.\n", filename.c_str());
+		my_printError("please use absolute path");
+		return;
+	}
+	else
+	  {
 		std::string tmp, variableName;
 		unsigned cat = 0;
 		while (getline(input, tmp))
@@ -207,6 +224,7 @@ void ROCParameter::initROCValuesFromFile(std::string filename)
 						}
 					}
 				}
+
 				else if (variableName == "std_csp")
 				{
 					double val;
@@ -216,34 +234,34 @@ void ROCParameter::initROCValuesFromFile(std::string filename)
 						std_csp.push_back(val);
 					}
 				}
-				else if (variableName == "noiseOffset")
-				{
-					double val;
-					iss.str(tmp);
-					while (iss >> val)
-					{
-						noiseOffset.push_back(val);
-						noiseOffset_proposed.push_back(val);
-					}
-				}
-				else if (variableName == "observedSynthesisNoise")
-				{
-					double val;
-					iss.str(tmp);
-					while (iss >> val)
-					{
-						observedSynthesisNoise.push_back(val);
-					}
-				}
-				else if (variableName == "std_NoiseOffset")
-				{
-					double val;
-					iss.str(tmp);
-					while (iss >> val)
-					{
-						std_NoiseOffset.push_back(val);
-					}
-				}
+				// else if (variableName == "noiseOffset")
+				// {
+				// 	double val;
+				// 	iss.str(tmp);
+				// 	while (iss >> val)
+				// 	{
+				// 		noiseOffset.push_back(val);
+				// 		noiseOffset_proposed.push_back(val);
+				// 	}
+				// }
+				// else if (variableName == "observedSynthesisNoise")
+				// {
+				// 	double val;
+				// 	iss.str(tmp);
+				// 	while (iss >> val)
+				// 	{
+				// 		observedSynthesisNoise.push_back(val);
+				// 	}
+				// }
+				// else if (variableName == "std_NoiseOffset")
+				// {
+				// 	double val;
+				// 	iss.str(tmp);
+				// 	while (iss >> val)
+				// 	{
+				// 		std_NoiseOffset.push_back(val);
+				// 	}
+				// }
 				else if (variableName == "covarianceMatrix")
 				{
 					if (tmp == "***") //end of matrix
@@ -259,18 +277,79 @@ void ROCParameter::initROCValuesFromFile(std::string filename)
 						mat.push_back(val);
 					}
 				}
+				else if (variableName == "mutation_prior_mean")
+				{
+					if (tmp == "***")
+					{
+						old_format = false;
+						mutation_prior_mean.resize(mutation_prior_mean.size() + 1);
+						cat++;
+					}
+					else if (tmp == "\n")
+						continue;
+					else
+					{
+						double val;
+						iss.str(tmp);
+						while (iss >> val)
+						{
+							mutation_prior_mean[cat - 1].push_back(val);
+						}
+					}
+				}
 				else if (variableName == "mutation_prior_sd")
 				{
-					iss.str(tmp);
-					iss >> mutation_prior_sd;
+					if (tmp == "***")
+					{
+						old_format = false;
+						mutation_prior_sd.resize(mutation_prior_sd.size() + 1);
+						cat++;
+					}
+					else if (tmp == "\n")
+						continue;
+					else
+					{
+						double val = 0.0;
+						iss.str(tmp);
+						if (!old_format)
+						{
+							while (iss >> val)
+							{
+								mutation_prior_sd[cat - 1].push_back(val);
+							}
+						}
+						else
+						{
+							mutation_prior_sd.resize(numMutationCategories);
+							for (int i = 0; i < numMutationCategories;i++)
+							{
+								mutation_prior_sd[i].resize(40);
+								for (int j = 0; j < 40; j++)
+								{
+									mutation_prior_sd[i][j] = val;
+								}
+							}
+						}
+					}
 				}
 			}
 		}
 	}
 	input.close();
-
+	if (!old_format)
+	{
+		mutation_prior_mean.resize(numMutationCategories);
+		for (int i = 0; i < numMutationCategories;i++)
+		{
+			mutation_prior_mean[i].resize(40);
+			for (int j = 0; j < 40; j++)
+			{
+				mutation_prior_mean[i][j] = 0.0;
+			}
+		}
+	}
 	//init other values
-	numAcceptForNoiseOffset.resize(obsPhiSets, 0);
+	//numAcceptForNoiseOffset.resize(obsPhiSets, 0);
 	bias_csp = 0;
 	proposedCodonSpecificParameter[dM].resize(numMutationCategories);
 	proposedCodonSpecificParameter[dEta].resize(numSelectionCategories);
@@ -298,46 +377,77 @@ void ROCParameter::writeROCRestartFile(std::string filename)
 	out.open(filename.c_str(), std::ofstream::app);
 	if (out.fail())
 		my_printError("Error opening file % to write restart file.\n", filename.c_str());
+		
 	else
 	{
 		std::ostringstream oss;
 		unsigned j;
-		oss << ">noiseOffset:\n";
-		for (j = 0; j < noiseOffset.size(); j++)
-		{
-			oss << noiseOffset[j];
-			if ((j + 1) % 10 == 0)
-				oss << "\n";
-			else
-				oss << " ";
-		}
-		if (j % 10 != 0)
-			oss << "\n";
+		// oss << ">noiseOffset:\n";
+		// for (j = 0; j < noiseOffset.size(); j++)
+		// {
+		// 	oss << noiseOffset[j];
+		// 	if ((j + 1) % 10 == 0)
+		// 		oss << "\n";
+		// 	else
+		// 		oss << " ";
+		// }
+		// if (j % 10 != 0)
+		// 	oss << "\n";
 
-		oss << ">observedSynthesisNoise:\n";
-		for (j = 0; j < observedSynthesisNoise.size(); j++)
-		{
-			oss << observedSynthesisNoise[j];
-			if ((j + 1) % 10 == 0)
-				oss << "\n";
-			else
-				oss << " ";
-		}
-		if (j % 10 != 0)
-			oss << "\n";
+		// oss << ">observedSynthesisNoise:\n";
+		// for (j = 0; j < observedSynthesisNoise.size(); j++)
+		// {
+		// 	oss << observedSynthesisNoise[j];
+		// 	if ((j + 1) % 10 == 0)
+		// 		oss << "\n";
+		// 	else
+		// 		oss << " ";
+		// }
+		// if (j % 10 != 0)
+		// 	oss << "\n";
 
-		oss << ">mutation_prior_sd:\n" << mutation_prior_sd << "\n";
-		oss << ">std_NoiseOffset:\n";
-		for (j = 0; j < std_NoiseOffset.size(); j++)
+		//oss << ">mutation_prior_sd:\n" << mutation_prior_sd << "\n";
+		oss << ">mutation_prior_mean:\n";
+		for (unsigned i = 0; i < mutation_prior_mean.size(); i++)
 		{
-			oss << std_NoiseOffset[j];
-			if ((j + 1) % 10 == 0)
+			oss << "***\n";
+			for (j = 0; j < mutation_prior_mean[i].size(); j++)
+			{
+				oss << mutation_prior_mean[i][j];
+				if ((j + 1) % 10 == 0)
+					oss << "\n";
+				else
+					oss << " ";
+			}
+			if (j % 10 != 0)
 				oss << "\n";
-			else
-				oss << " ";
 		}
-		if (j % 10 != 0)
-			oss << "\n";
+		oss << ">mutation_prior_sd:\n";
+		for (unsigned i = 0; i < mutation_prior_sd.size(); i++)
+		{
+			oss << "***\n";
+			for (j = 0; j < mutation_prior_sd[i].size(); j++)
+			{
+				oss << mutation_prior_sd[i][j];
+				if ((j + 1) % 10 == 0)
+					oss << "\n";
+				else
+					oss << " ";
+			}
+			if (j % 10 != 0)
+				oss << "\n";
+		}
+		// oss << ">std_NoiseOffset:\n";
+		// for (j = 0; j < std_NoiseOffset.size(); j++)
+		// {
+		// 	oss << std_NoiseOffset[j];
+		// 	if ((j + 1) % 10 == 0)
+		// 		oss << "\n";
+		// 	else
+		// 		oss << " ";
+		// }
+		// if (j % 10 != 0)
+		// 	oss << "\n";
 		oss << ">std_csp:\n";
 		for (j = 0; j < std_csp.size(); j++)
 		{
@@ -409,14 +519,20 @@ void ROCParameter::initFromRestartFile(std::string filename)
 }
 
 
-void ROCParameter::initAllTraces(unsigned samples, unsigned num_genes)
+void ROCParameter::initAllTraces(unsigned samples, unsigned num_genes, bool estimateSynthesisRate)
 {
 	traces.initializeROCTrace(samples, num_genes, numMutationCategories, numSelectionCategories, numParam,
-						 numMixtures, categories, maxGrouping, obsPhiSets,currentSynthesisRateLevel[0],mixtureAssignment);
+						 numMixtures, categories, maxGrouping, obsPhiSets,currentSynthesisRateLevel[0],mixtureAssignment,estimateSynthesisRate);
 }
 
 
-void ROCParameter::initMutationCategories(std::vector<std::string> files, unsigned numCategories)
+//' @name initMutationCategories
+//' @title Initialize values for mutation CSP. File should be of comma-separated with header. Three columns should be of order Amino_acid,Codon,Value
+//' @param files list of files containing starting values. Number of files should equal the number of categories.
+//' @param numCategories number of mutation categories (should be less than or equal to number of mixtures)
+//' @param fix Can use this parameter to fix mutation at current values (won't change over course of MCMC run)
+
+void ROCParameter::initMutationCategories(std::vector<std::string> files, unsigned numCategories, bool fix)
 {
 	for (unsigned category = 0; category < numCategories; category++)
 	{
@@ -424,12 +540,17 @@ void ROCParameter::initMutationCategories(std::vector<std::string> files, unsign
 		std::ifstream currentFile;
 		currentFile.open(files[category].c_str());
 		if (currentFile.fail())
+		{
 			my_printError("Error opening file % to initialize mutation values.\n", category);
+			my_printError("please use absolute path");
+			return;
+		}
+
 		else
 		{
 			std::string tmp;
 			currentFile >> tmp; //The first line is a header (Amino Acid, Codon, Value, Std_deviation)
-
+			fix_dM = fix;
 			while (currentFile >> tmp)
 			{
 				//Get the Codon and Index
@@ -451,8 +572,13 @@ void ROCParameter::initMutationCategories(std::vector<std::string> files, unsign
 	} //END OF A CATEGORY/FILE
 }
 
+//' @name initSelectionCategories
+//' @title Initialize values for selection CSP. File should be of comma-separated with header. Three columns should be of order Amino_acid,Codon,Value
+//' @param files list of files containing starting values. Number of files should equal the number of categories.
+//' @param numCategories number of mutation categories (should be less than or equal to number of mixtures)
+//' @param fix Can use this parameter to fix selection at current values (won't change over course of MCMC run)
 
-void ROCParameter::initSelectionCategories(std::vector<std::string> files, unsigned numCategories)
+void ROCParameter::initSelectionCategories(std::vector<std::string> files, unsigned numCategories, bool fix)
 {
 	for (unsigned category = 0; category < numCategories; category++)
 	{
@@ -460,12 +586,17 @@ void ROCParameter::initSelectionCategories(std::vector<std::string> files, unsig
 		std::ifstream currentFile;
 		currentFile.open(files[category].c_str());
 		if (currentFile.fail())
+		{
 			my_printError("Error opening file % to initialize selection values.\n", category);
+			my_printError("please use absolute path");
+			return;
+		}
+
 		else
 		{
 			std::string tmp;
 			currentFile >> tmp; //The first line is a header (Amino Acid, Codon, Value, Std_deviation)
-
+			fix_dEta = fix;
 			while (currentFile >> tmp)
 			{
 				//Get the Codon and Index
@@ -495,22 +626,22 @@ void ROCParameter::initSelectionCategories(std::vector<std::string> files, unsig
 //---------- Trace Functions -----------//
 //--------------------------------------//
 
-void ROCParameter::updateObservedSynthesisNoiseTraces(unsigned sample)
-{
-	for (unsigned i = 0; i < observedSynthesisNoise.size(); i++)
-	{
-		traces.updateObservedSynthesisNoiseTrace(i, sample, observedSynthesisNoise[i]);
-	}
-}
+// void ROCParameter::updateObservedSynthesisNoiseTraces(unsigned sample)
+// {
+// 	for (unsigned i = 0; i < observedSynthesisNoise.size(); i++)
+// 	{
+// 		traces.updateObservedSynthesisNoiseTrace(i, sample, observedSynthesisNoise[i]);
+// 	}
+// }
 
 
-void ROCParameter::updateNoiseOffsetTraces(unsigned sample)
-{
-	for (unsigned i = 0; i < noiseOffset.size(); i++)
-	{
-		traces.updateSynthesisOffsetTrace(i, sample, noiseOffset[i]);
-	}
-}
+// void ROCParameter::updateNoiseOffsetTraces(unsigned sample)
+// {
+// 	for (unsigned i = 0; i < noiseOffset.size(); i++)
+// 	{
+// 		traces.updateSynthesisOffsetTrace(i, sample, noiseOffset[i]);
+// 	}
+// }
 
 void ROCParameter::updateCodonSpecificParameterTrace(unsigned sample, std::string grouping)
 {
@@ -519,8 +650,31 @@ void ROCParameter::updateCodonSpecificParameterTrace(unsigned sample, std::strin
 }
 
 
+//' @name fixDM
+//' @title Fix the value of mutation its current value
 
+void ROCParameter::fixDM()
+{
+	fix_dM = true;
+}
 
+//' @name fixDEta
+//' @title Fix the value of selection its current value
+
+void ROCParameter::fixDEta()
+{
+	fix_dEta = true;
+}
+
+bool ROCParameter::isDMFixed()
+{
+	return fix_dM;
+}
+
+bool ROCParameter::isDEtaFixed()
+{
+	return fix_dEta;
+}
 
 //------------------------------------------//
 //---------- Covariance Functions ----------//
@@ -543,79 +697,79 @@ CovarianceMatrix& ROCParameter::getCovarianceMatrixForAA(std::string aa)
 //------------------------------------------------------//
 
 
-double ROCParameter::getObservedSynthesisNoise(unsigned index)
-{
-	return observedSynthesisNoise[index];
-}
+// double ROCParameter::getObservedSynthesisNoise(unsigned index)
+// {
+// 	return observedSynthesisNoise[index];
+// }
 
 
-void ROCParameter::setObservedSynthesisNoise(unsigned index, double se)
-{
-	observedSynthesisNoise[index] = se;
-}
+// void ROCParameter::setObservedSynthesisNoise(unsigned index, double se)
+// {
+// 	observedSynthesisNoise[index] = se;
+// }
 
 
 
 
 
-//-------------------------------------------//
-//---------- noiseOffset Functions ----------//
-//-------------------------------------------//
+// //-------------------------------------------//
+// //---------- noiseOffset Functions ----------//
+// //-------------------------------------------//
 
 
-double ROCParameter::getNoiseOffset(unsigned index, bool proposed)
-{
-	return (proposed ? noiseOffset_proposed[index] : noiseOffset[index]);
-}
+// double ROCParameter::getNoiseOffset(unsigned index, bool proposed)
+// {
+// 	return (proposed ? noiseOffset_proposed[index] : noiseOffset[index]);
+// }
 
 
-double ROCParameter::getCurrentNoiseOffsetProposalWidth(unsigned index)
-{
-	return std_NoiseOffset[index];
-}
+// double ROCParameter::getCurrentNoiseOffsetProposalWidth(unsigned index)
+// {
+// 	return std_NoiseOffset[index];
+// }
 
 
-void ROCParameter::proposeNoiseOffset()
-{
-	for (unsigned i = 0; i < getNumObservedPhiSets(); i++)
-	{
-		noiseOffset_proposed[i] = randNorm(noiseOffset[i], std_NoiseOffset[i]);
-	}
-}
+// void ROCParameter::proposeNoiseOffset()
+// {
+// 	for (unsigned i = 0; i < getNumObservedPhiSets(); i++)
+// 	{
+// 		noiseOffset_proposed[i] = randNorm(noiseOffset[i], std_NoiseOffset[i]);
+// 	}
+// }
 
 
-void ROCParameter::setNoiseOffset(unsigned index, double _noiseOffset)
-{
-	noiseOffset[index] = _noiseOffset;
-}
+// void ROCParameter::setNoiseOffset(unsigned index, double _noiseOffset)
+// {
+// 	noiseOffset[index] = _noiseOffset;
+// }
 
 
-void ROCParameter::updateNoiseOffset(unsigned index)
-{
-	noiseOffset[index] = noiseOffset_proposed[index];
-	numAcceptForNoiseOffset[index]++;
-}
+// void ROCParameter::updateNoiseOffset(unsigned index)
+// {
+// 	noiseOffset[index] = noiseOffset_proposed[index];
+// 	numAcceptForNoiseOffset[index]++;
+// }
 
 
 //-----------------------------------//
 //---------- Noise Functions --------//
 //-----------------------------------//
 
-void ROCParameter::setInitialValuesForSepsilon(std::vector<double> seps)
-{
-	if (seps.size() == observedSynthesisNoise.size())
-	{
-		for (unsigned i = 0; i < observedSynthesisNoise.size(); i++)
-		{
-			observedSynthesisNoise[i] = seps[i];
-		}
-	}
-	else
-	{
-		my_printError("ROCParameter::setInitialValuesForSepsilon number of initial values (%) does not match number of expression sets (%)",
-					  seps.size(), observedSynthesisNoise.size());
-	}
-}
+// void ROCParameter::setInitialValuesForSepsilon(std::vector<double> seps)
+// {
+// 	if (seps.size() == observedSynthesisNoise.size())
+// 	{
+// 		for (unsigned i = 0; i < observedSynthesisNoise.size(); i++)
+// 		{
+// 			observedSynthesisNoise[i] = seps[i];
+// 		}
+// 	}
+// 	else
+// 	{
+// 		my_printError("ROCParameter::setInitialValuesForSepsilon number of initial values (%) does not match number of expression sets (%)",
+// 					  seps.size(), observedSynthesisNoise.size());
+// 	}
+// }
 
 
 //-----------------------------------//
@@ -630,7 +784,6 @@ double ROCParameter::getCurrentCodonSpecificProposalWidth(unsigned aa)
 	return std_csp[aaStart];
 }
 
-
 /* Cedric: I decided to use a normal distribution to propose Sphi and phi instead of a lognormal because:
  * 1. It is a symmetric distribution and you therefore do not have to account for the unsymmetry in jump probabilities
  * 2. The one log and exp operation that have to be performed per parameter are cheaper than the operations necessary to draw from a lognormal
@@ -653,65 +806,169 @@ void ROCParameter::proposeCodonSpecificParameter()
 		}
 
 		std::vector<double> covaryingNums;
-		covaryingNums = covarianceMatrix[SequenceSummary::AAToAAIndex(aa)].transformIidNumbersIntoCovaryingNumbers(
-				iidProposed);
+		covaryingNums = covarianceMatrix[SequenceSummary::AAToAAIndex(aa)].transformIidNumbersIntoCovaryingNumbers(iidProposed);
 		for (unsigned i = 0; i < numMutationCategories; i++)
 		{
 			for (unsigned j = i * numCodons, l = aaStart; j < (i * numCodons) + numCodons; j++, l++)
 			{
-				
-				proposedCodonSpecificParameter[dM][i][l] = currentCodonSpecificParameter[dM][i][l] + covaryingNums[j];
-				
+				if (fix_dM)
+				{
+					proposedCodonSpecificParameter[dM][i][l] = currentCodonSpecificParameter[dM][i][l];
+				}
+				else if (propose_by_prior)
+				{
+					proposedCodonSpecificParameter[dM][i][l] = randNorm(mutation_prior_mean[i][l],mutation_prior_sd[i][l]);
+				}
+				else
+				{
+					proposedCodonSpecificParameter[dM][i][l] = currentCodonSpecificParameter[dM][i][l] + covaryingNums[j];
+				}
 			}
 		}
 		for (unsigned i = 0; i < numSelectionCategories; i++)
 		{
 			for (unsigned j = i * numCodons, l = aaStart; j < (i * numCodons) + numCodons; j++, l++)
 			{
-				proposedCodonSpecificParameter[dEta][i][l] = currentCodonSpecificParameter[dEta][i][l]
+				if (fix_dEta)
+				{
+					proposedCodonSpecificParameter[dEta][i][l] = currentCodonSpecificParameter[dEta][i][l];
+				}
+				else
+				{
+					proposedCodonSpecificParameter[dEta][i][l] = currentCodonSpecificParameter[dEta][i][l]
 												   + covaryingNums[(numMutationCategories * numCodons) + j];
+				}
 			}
 		}
 	}
 }
 
+void ROCParameter::setProposeByPrior(bool _propose_by_prior)
+{
+	propose_by_prior = _propose_by_prior;
+}
+
+
+void ROCParameter::completeUpdateCodonSpecificParameter()
+{
+	for (std::string grouping : CSPToUpdate)
+	{
+		unsigned aaStart, aaEnd;
+		SequenceSummary::AAToCodonRange(grouping, aaStart, aaEnd, true);
+		unsigned aaIndex = SequenceSummary::aaToIndex.find(grouping)->second;
+		numAcceptForCodonSpecificParameters[aaIndex]++;
+		if (!fix_dM)
+		{
+			for (unsigned k = 0u; k < numMutationCategories; k++)
+			{
+				for (unsigned i = aaStart; i < aaEnd; i++)
+				{
+					currentCodonSpecificParameter[dM][k][i] = proposedCodonSpecificParameter[dM][k][i];
+				}
+			}
+		}
+		if (!fix_dEta)
+		{
+			for (unsigned k = 0u; k < numSelectionCategories; k++)
+			{
+				for (unsigned i = aaStart; i < aaEnd; i++)
+				{
+					currentCodonSpecificParameter[dEta][k][i] = proposedCodonSpecificParameter[dEta][k][i];
+				}
+			}
+		}
+	}
+	CSPToUpdate.clear();
+}
+
 
 void ROCParameter::updateCodonSpecificParameter(std::string grouping)
 {
+	//CSPToUpdate.push_back(grouping);
 	unsigned aaStart, aaEnd;
 	SequenceSummary::AAToCodonRange(grouping, aaStart, aaEnd, true);
 	unsigned aaIndex = SequenceSummary::aaToIndex.find(grouping)->second;
 	numAcceptForCodonSpecificParameters[aaIndex]++;
-
-	for (unsigned k = 0u; k < numMutationCategories; k++)
+	if (!fix_dM)
 	{
-		for (unsigned i = aaStart; i < aaEnd; i++)
+		for (unsigned k = 0u; k < numMutationCategories; k++)
 		{
-			currentCodonSpecificParameter[dM][k][i] = proposedCodonSpecificParameter[dM][k][i];
+			for (unsigned i = aaStart; i < aaEnd; i++)
+			{
+				currentCodonSpecificParameter[dM][k][i] = proposedCodonSpecificParameter[dM][k][i];
+			}
 		}
 	}
-	for (unsigned k = 0u; k < numSelectionCategories; k++)
+	if (!fix_dEta)
 	{
-		for (unsigned i = aaStart; i < aaEnd; i++)
+		for (unsigned k = 0u; k < numSelectionCategories; k++)
 		{
-			currentCodonSpecificParameter[dEta][k][i] = proposedCodonSpecificParameter[dEta][k][i];
+			for (unsigned i = aaStart; i < aaEnd; i++)
+			{
+				currentCodonSpecificParameter[dEta][k][i] = proposedCodonSpecificParameter[dEta][k][i];
+			}
 		}
 	}
 }
 
-
 //-------------------------------------//
 //---------- Prior Functions ----------//
 //-------------------------------------//
 
+std::vector<std::vector<double>> ROCParameter::getMutationPriorMean()
+{
+	return mutation_prior_mean;
+}
 
-double ROCParameter::getMutationPriorStandardDeviation()
+std::vector<std::vector<double>> ROCParameter::getMutationPriorStandardDeviation()
 {
 	return mutation_prior_sd;
 }
 
 
-void ROCParameter::setMutationPriorStandardDeviation(double _mutation_prior_sd)
+std::vector<double> ROCParameter::getMutationPriorMeanForCategory(unsigned category)
+{
+	return mutation_prior_mean[category];
+}
+
+std::vector<double> ROCParameter::getMutationPriorStandardDeviationForCategory(unsigned category)
+{
+	return mutation_prior_sd[category];
+}
+
+void ROCParameter::getMutationPriorMeanForCategoryForGroup(unsigned category, std::string aa, double *returnSet)
+{
+	unsigned aaStart, aaEnd;
+	SequenceSummary::AAToCodonRange(aa, aaStart, aaEnd, true);
+	std::vector<double> mutation_prior_mean_category = mutation_prior_mean[category];
+	unsigned j = 0u;
+	for (unsigned i = aaStart; i < aaEnd; i++, j++)
+	{
+		returnSet[j] = mutation_prior_mean_category[i];
+	}
+}
+
+
+void ROCParameter::getMutationPriorStandardDeviationForCategoryForGroup(unsigned category, std::string aa, double *returnSet)
+{
+	unsigned aaStart, aaEnd;
+	SequenceSummary::AAToCodonRange(aa, aaStart, aaEnd, true);
+	std::vector<double> mutation_prior_sd_category = mutation_prior_sd[category];
+	unsigned j = 0u;
+	for (unsigned i = aaStart; i < aaEnd; i++, j++)
+	{
+		returnSet[j] = mutation_prior_sd_category[i];
+	}
+}
+
+
+void ROCParameter::setMutationPriorMean(std::vector<std::vector<double>> _mutation_prior_mean)
+{
+	mutation_prior_mean = _mutation_prior_mean;
+}
+
+
+void ROCParameter::setMutationPriorStandardDeviation(std::vector<std::vector<double>> _mutation_prior_sd)
 {
 	mutation_prior_sd = _mutation_prior_sd;
 }
@@ -725,52 +982,52 @@ void ROCParameter::setMutationPriorStandardDeviation(double _mutation_prior_sd)
 //------------------------------------------------------------------//
 
 
-double ROCParameter::getNoiseOffsetPosteriorMean(unsigned index, unsigned samples)
-{
-	double posteriorMean = 0.0;
-	std::vector<double> NoiseOffsetTrace = traces.getSynthesisOffsetTrace(index);
-	unsigned traceLength = lastIteration;
+// double ROCParameter::getNoiseOffsetPosteriorMean(unsigned index, unsigned samples)
+// {
+// 	double posteriorMean = 0.0;
+// 	std::vector<double> NoiseOffsetTrace = traces.getSynthesisOffsetTrace(index);
+// 	unsigned traceLength = lastIteration;
 
-	if (samples > traceLength)
-	{
-		my_printError("Warning in ROCParameter::getNoiseOffsetPosteriorMean throws: Number of anticipated samples ");
-		my_printError("(%) is greater than the length of the available trace (%). Whole trace is used for posterior estimate! \n", samples, traceLength);
+// 	if (samples > traceLength)
+// 	{
+// 		my_printError("Warning in ROCParameter::getNoiseOffsetPosteriorMean throws: Number of anticipated samples ");
+// 		my_printError("(%) is greater than the length of the available trace (%). Whole trace is used for posterior estimate! \n", samples, traceLength);
 
-		samples = traceLength;
-	}
-	unsigned start = traceLength - samples;
+// 		samples = traceLength;
+// 	}
+// 	unsigned start = traceLength - samples;
 
-	for (unsigned i = start; i < traceLength; i++)
-		posteriorMean += NoiseOffsetTrace[i];
+// 	for (unsigned i = start; i < traceLength; i++)
+// 		posteriorMean += NoiseOffsetTrace[i];
 
-	return posteriorMean / (double)samples;
-}
+// 	return posteriorMean / (double)samples;
+// }
 
 
-double ROCParameter::getNoiseOffsetVariance(unsigned index, unsigned samples, bool unbiased)
-{
-	std::vector<double> NoiseOffsetTrace = traces.getSynthesisOffsetTrace(index);
-	unsigned traceLength = lastIteration;
-	if (samples > traceLength)
-	{
-		my_printError("Warning in ROCParameter::getNoiseOffsetVariance throws: Number of anticipated samples ");
-		my_printError("(%) is greater than the length of the available trace (%). Whole trace is used for posterior estimate! \n", samples, traceLength);
+// double ROCParameter::getNoiseOffsetVariance(unsigned index, unsigned samples, bool unbiased)
+// {
+// 	std::vector<double> NoiseOffsetTrace = traces.getSynthesisOffsetTrace(index);
+// 	unsigned traceLength = lastIteration;
+// 	if (samples > traceLength)
+// 	{
+// 		my_printError("Warning in ROCParameter::getNoiseOffsetVariance throws: Number of anticipated samples ");
+// 		my_printError("(%) is greater than the length of the available trace (%). Whole trace is used for posterior estimate! \n", samples, traceLength);
 
-		samples = traceLength;
-	}
-	double posteriorMean = getNoiseOffsetPosteriorMean(index, samples);
+// 		samples = traceLength;
+// 	}
+// 	double posteriorMean = getNoiseOffsetPosteriorMean(index, samples);
 
-	double posteriorVariance = 0.0;
+// 	double posteriorVariance = 0.0;
 
-	unsigned start = traceLength - samples;
-	for (unsigned i = start; i < traceLength; i++)
-	{
-		double difference = NoiseOffsetTrace[i] - posteriorMean;
-		posteriorVariance += difference * difference;
-	}
-	double normalizationTerm = unbiased ? (1 / ((double)samples - 1.0)) : (1 / (double)samples);
-	return normalizationTerm * posteriorVariance;
-}
+// 	unsigned start = traceLength - samples;
+// 	for (unsigned i = start; i < traceLength; i++)
+// 	{
+// 		double difference = NoiseOffsetTrace[i] - posteriorMean;
+// 		posteriorVariance += difference * difference;
+// 	}
+// 	double normalizationTerm = unbiased ? (1 / ((double)samples - 1.0)) : (1 / (double)samples);
+// 	return normalizationTerm * posteriorVariance;
+// }
 
 
 
@@ -781,23 +1038,23 @@ double ROCParameter::getNoiseOffsetVariance(unsigned index, unsigned samples, bo
 //----------------------------------------------//
 
 
-void ROCParameter::adaptNoiseOffsetProposalWidth(unsigned adaptationWidth, bool adapt)
-{
-	for (unsigned i = 0; i < getNumObservedPhiSets(); i++)
-	{
-		double acceptanceLevel = numAcceptForNoiseOffset[i] / (double)adaptationWidth;
-		traces.updateSynthesisOffsetAcceptanceRateTrace(i, acceptanceLevel);
-		if (adapt)
-		{
-			if (acceptanceLevel < 0.2)
-				std_NoiseOffset[i] *= 0.8;
-			if (acceptanceLevel > 0.3)
-				std_NoiseOffset[i] *= 1.2;
+// void ROCParameter::adaptNoiseOffsetProposalWidth(unsigned adaptationWidth, bool adapt)
+// {
+// 	for (unsigned i = 0; i < getNumObservedPhiSets(); i++)
+// 	{
+// 		double acceptanceLevel = numAcceptForNoiseOffset[i] / (double)adaptationWidth;
+// 		traces.updateSynthesisOffsetAcceptanceRateTrace(i, acceptanceLevel);
+// 		if (adapt)
+// 		{
+// 			if (acceptanceLevel < 0.2)
+// 				std_NoiseOffset[i] *= 0.8;
+// 			if (acceptanceLevel > 0.3)
+// 				std_NoiseOffset[i] *= 1.2;
 
-			numAcceptForNoiseOffset[i] = 0u;
-		}
-	}
-}
+// 			numAcceptForNoiseOffset[i] = 0u;
+// 		}
+// 	}
+// }
 
 
 
@@ -808,15 +1065,15 @@ void ROCParameter::adaptNoiseOffsetProposalWidth(unsigned adaptationWidth, bool
 //-------------------------------------//
 
 
-void ROCParameter::setNumObservedPhiSets(unsigned _phiGroupings)
-{
-	obsPhiSets = _phiGroupings;
-	noiseOffset.resize(obsPhiSets, 0.1);
-	noiseOffset_proposed.resize(obsPhiSets, 0.1);
-	std_NoiseOffset.resize(obsPhiSets, 0.1);
-	numAcceptForNoiseOffset.resize(obsPhiSets, 0);
-	observedSynthesisNoise.resize(obsPhiSets, 1.0);
-}
+// void ROCParameter::setNumObservedPhiSets(unsigned _phiGroupings)
+// {
+// 	obsPhiSets = _phiGroupings;
+// 	noiseOffset.resize(obsPhiSets, 0.1);
+// 	noiseOffset_proposed.resize(obsPhiSets, 0.1);
+// 	std_NoiseOffset.resize(obsPhiSets, 0.1);
+// 	numAcceptForNoiseOffset.resize(obsPhiSets, 0);
+// 	observedSynthesisNoise.resize(obsPhiSets, 1.0);
+// }
 
 
 void ROCParameter::getParameterForCategory(unsigned category, unsigned paramType, std::string aa, bool proposal,
@@ -959,6 +1216,33 @@ void ROCParameter::initSelection(std::vector<double> selectionValues, unsigned m
     }
 }
 
+void ROCParameter::setMutationPriorMeanR(std::vector<double> _mutation_prior_mean)
+{
+	unsigned index = 0;
+	for (unsigned i = 0; i < numMutationCategories; i++)
+	{
+		for (unsigned j = 0; j < 40; j++,index++)
+		{
+			mutation_prior_mean[i][j] = _mutation_prior_mean[index];
+		}
+	}
+}
+
+
+
+void ROCParameter::setMutationPriorStandardDeviationR(std::vector<double> _mutation_prior_sd)
+{
+	unsigned index = 0;
+	for (unsigned i = 0; i < numMutationCategories; i++)
+	{
+		for (unsigned j = 0; j < 40; j++,index++)
+		{
+			mutation_prior_sd[i][j] = _mutation_prior_sd[index];
+		}
+	}
+}
+
+
 
 // -----------------------------------//
 // ---------- CSP Functions ----------//
@@ -1013,6 +1297,8 @@ void ROCParameter::setCurrentSelectionParameter(std::vector<std::vector<double>>
 }
 
 
+
+
 // ------------------------------------------------------------------//
 // ---------- Posterior, Variance, and Estimates Functions ----------//
 // ------------------------------------------------------------------//
