@@ -44,7 +44,7 @@ Trace::Trace(unsigned _numCodonSpecificParamTypes)
 
 void Trace::initializeSharedTraces(unsigned samples, unsigned num_genes, unsigned numSelectionCategories,
 	unsigned numMixtures, std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, std::vector<double> init_phi,
-	std::vector<unsigned> init_mix_assign)
+	std::vector<unsigned> init_mix_assign, unsigned numObservedPhiSets, bool estimateSynthesisRate)
 {
 	my_print("maxGrouping: %\n", maxGrouping);
 
@@ -53,10 +53,12 @@ void Trace::initializeSharedTraces(unsigned samples, unsigned num_genes, unsigne
 	initStdDevSynthesisRateTrace(numSelectionCategories, samples);
 	initSynthesisRateAcceptanceRateTrace(num_genes, numSelectionCategories);
 	codonSpecificAcceptanceRateTrace.resize(maxGrouping);
-	initSynthesisRateTrace(samples, num_genes, numSelectionCategories,init_phi);
+	initSynthesisRateTrace(samples, num_genes, numSelectionCategories,init_phi,estimateSynthesisRate);
+	
 	initMixtureAssignmentTrace(samples, num_genes,init_mix_assign);
 	initMixtureProbabilitiesTrace(samples, numMixtures);
-
+	initSynthesisOffsetTrace(samples, numObservedPhiSets);
+	initObservedSynthesisNoiseTrace(samples, numObservedPhiSets);
 	categories = &_categories;
 }
 
@@ -83,7 +85,7 @@ void Trace::initSynthesisRateAcceptanceRateTrace(unsigned num_genes, unsigned nu
 }
 
 
-void Trace::initSynthesisRateTrace(unsigned samples, unsigned num_genes, unsigned numSynthesisRateCategories,std::vector<double> init_phi)
+void Trace::initSynthesisRateTrace(unsigned samples, unsigned num_genes, unsigned numSynthesisRateCategories,std::vector<double> init_phi, bool estimateSynthesisRate)
 {
 	synthesisRateTrace.resize(numSynthesisRateCategories);
 	for (unsigned category = 0; category < numSynthesisRateCategories; category++)
@@ -91,8 +93,16 @@ void Trace::initSynthesisRateTrace(unsigned samples, unsigned num_genes, unsigne
 		synthesisRateTrace[category].resize(num_genes);
 		for (unsigned i = 0; i < num_genes; i++)
 		{
-			std::vector<float> tempExpr(samples, init_phi[i]);
-			synthesisRateTrace[category][i] = tempExpr;
+			// if (estimateSynthesisRate)
+			// {
+				std::vector<float> tempExpr(samples, init_phi[i]);
+				synthesisRateTrace[category][i] = tempExpr;
+			// }
+			// else
+			// {
+			// 	synthesisRateTrace[category][i].resize(1);
+			// 	synthesisRateTrace[category][i][0] = init_phi[i];
+			// }
 		}
 	}
 }
@@ -154,15 +164,6 @@ void Trace::initCodonSpecificParameterTrace(unsigned samples, unsigned numCatego
 
 
 
-
-
-
-
-
-//----------------------------------//
-//---------- ROC Specific ----------//
-//----------------------------------//
-
 void Trace::initSynthesisOffsetTrace(unsigned samples, unsigned numPhiGroupings)
 {
 	synthesisOffsetTrace.resize(numPhiGroupings);
@@ -185,9 +186,30 @@ void Trace::initObservedSynthesisNoiseTrace(unsigned samples, unsigned numPhiGro
 }
 
 
+//----------------------------------//
+//--------- PANSE Specific ---------//
+//----------------------------------//
+void Trace::initPartitionFunctionTrace(unsigned samples, unsigned numPartitionFunctionsGroupings)
+{
+    partitionFunctionTrace.resize(numPartitionFunctionsGroupings);
+    for (unsigned i = 0; i < numPartitionFunctionsGroupings; i++)
+    {
+        partitionFunctionTrace[i].resize(samples);
+    }
+}
 
 
 
+//------------------------------------//
+//----------- FONSE Specific ---------//
+//------------------------------------//
+
+void Trace::initInitiationCostTrace(unsigned samples)
+{
+	initiationCostTrace.resize(samples);	
+}
+
+
 //----------------------------------------------------//
 //---------- Model Initialization Functions ----------//
 //----------------------------------------------------//
@@ -200,11 +222,11 @@ void Trace::initObservedSynthesisNoiseTrace(unsigned samples, unsigned numPhiGro
 
 void Trace::initializePATrace(unsigned samples, unsigned num_genes, unsigned numAlphaCategories,
 	unsigned numLambdaPrimeCategories, unsigned numParam, unsigned numMixtures,
-	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, std::vector<double> init_phi,
-	std::vector<unsigned> init_mix_assign)
+	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, unsigned numObservedPhiSets, std::vector<double> init_phi,
+	std::vector<unsigned> init_mix_assign,bool estimateSynthesisRate)
 {
 	initializeSharedTraces(samples, num_genes, numLambdaPrimeCategories, numMixtures,
-		_categories, maxGrouping, init_phi, init_mix_assign);
+		_categories, maxGrouping, init_phi, init_mix_assign,numObservedPhiSets,estimateSynthesisRate);
 
 	// See Note 1) above.
 	initCodonSpecificParameterTrace(samples, numAlphaCategories,  numParam, 0u); // alp
@@ -216,44 +238,53 @@ void Trace::initializePATrace(unsigned samples, unsigned num_genes, unsigned num
 void Trace::initializeROCTrace(unsigned samples, unsigned num_genes, unsigned numMutationCategories,
 	unsigned numSelectionCategories, unsigned numParam, unsigned numMixtures,
 	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, unsigned numObservedPhiSets,std::vector<double> init_phi,
-	std::vector<unsigned> init_mix_assign)
+	std::vector<unsigned> init_mix_assign,bool estimateSynthesisRate)
 {
-	initializeSharedTraces(samples, num_genes, numSelectionCategories, numMixtures, _categories, maxGrouping,init_phi,init_mix_assign);
+	initializeSharedTraces(samples, num_genes, numSelectionCategories, numMixtures, _categories, maxGrouping,init_phi,init_mix_assign,estimateSynthesisRate);
+
 
 	// See Note 1) above.
 	initCodonSpecificParameterTrace(samples, numMutationCategories, numParam, 0u); // dM
 	initCodonSpecificParameterTrace(samples, numSelectionCategories, numParam, 1u); // dEta
 
-	initSynthesisOffsetTrace(samples, numObservedPhiSets);
-	initObservedSynthesisNoiseTrace(samples, numObservedPhiSets);
+	// initSynthesisOffsetTrace(samples, numObservedPhiSets);
+	// initObservedSynthesisNoiseTrace(samples, numObservedPhiSets);
 }
 
 
 void Trace::initializeFONSETrace(unsigned samples, unsigned num_genes, unsigned numMutationCategories,
 	unsigned numSelectionCategories, unsigned numParam, unsigned numMixtures,
-	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, std::vector<double> init_phi,
-	std::vector<unsigned> init_mix_assign)
+	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, unsigned numObservedPhiSets,std::vector<double> init_phi,
+	std::vector<unsigned> init_mix_assign,bool estimateSynthesisRate)
 {
 	initializeSharedTraces(samples, num_genes, numSelectionCategories, numMixtures,
-		 _categories, maxGrouping,init_phi,init_mix_assign);
+		 _categories, maxGrouping,init_phi,init_mix_assign,numObservedPhiSets,estimateSynthesisRate);
 
 	// See Note 1) above.
 	initCodonSpecificParameterTrace(samples, numMutationCategories, numParam, 0u); // dM
 	initCodonSpecificParameterTrace(samples, numSelectionCategories, numParam, 1u); // dOmega
+	initInitiationCostTrace(samples);
 }
 
 
 void Trace::initializePANSETrace(unsigned samples, unsigned num_genes, unsigned numAlphaCategories,
 	unsigned numLambdaPrimeCategories, unsigned numParam, unsigned numMixtures,
-	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, std::vector<double> init_phi,
-	std::vector<unsigned> init_mix_assign)
+	std::vector<mixtureDefinition> &_categories, unsigned maxGrouping, unsigned numObservedPhiSets,std::vector<double> init_phi,
+	std::vector<unsigned> init_mix_assign, bool estimateSynthesisRate)
 {
+    numCodonSpecificParamTypes = 3;
+    codonSpecificParameterTrace.resize(numCodonSpecificParamTypes);
+
 	initializeSharedTraces(samples, num_genes, numLambdaPrimeCategories, numMixtures,
-		_categories, maxGrouping,init_phi,init_mix_assign);
+		_categories, maxGrouping,init_phi,init_mix_assign,numObservedPhiSets,estimateSynthesisRate);
 
 	// See Note 1) above.
 	initCodonSpecificParameterTrace(samples, numAlphaCategories,  numParam, 0u); //alp
 	initCodonSpecificParameterTrace(samples, numLambdaPrimeCategories, numParam, 1u); //lmPri
+    initCodonSpecificParameterTrace(samples, numAlphaCategories, numParam, 2u); //nseRate
+    initPartitionFunctionTrace(samples, numMixtures);
+
+    nseSpecificAcceptanceRateTrace.resize(maxGrouping);
 }
 
 
@@ -320,6 +351,14 @@ std::vector<double> Trace::getCodonSpecificAcceptanceRateTraceForAA(std::string
 	return codonSpecificAcceptanceRateTrace[aaIndex];
 }
 
+std::vector<double> Trace::getCodonSpecificAcceptanceRateTraceForCodon(std::string codon)
+{
+	codon[0] = (char)std::toupper(codon[0]);
+	unsigned codonIndex = SequenceSummary::codonToIndexWithReference.find(codon)->second;
+	return codonSpecificAcceptanceRateTrace[codonIndex];
+}
+
+
 
 std::vector<float> Trace::getSynthesisRateTraceForGene(unsigned geneIndex)
 {
@@ -374,6 +413,11 @@ std::vector<std::vector<double>> Trace::getCodonSpecificAcceptanceRateTrace()
 	return codonSpecificAcceptanceRateTrace;
 }
 
+std::vector<std::vector<double>> Trace::getNseRateSpecificAcceptanceRateTrace()
+{
+	return nseSpecificAcceptanceRateTrace;
+}
+
 
 unsigned Trace::getSynthesisRateCategory(unsigned mixtureElement)
 {
@@ -449,7 +493,7 @@ std::vector<std::vector<double>> Trace::getSynthesisOffsetAcceptanceRateTrace()
 	return synthesisOffsetAcceptanceRateTrace;
 }
 
-
+//TO DO: How does this work for PA/PANSE
 unsigned Trace::getCodonSpecificCategory(unsigned mixtureElement, unsigned paramType)
 {
 	unsigned rv = 0;
@@ -460,6 +504,9 @@ unsigned Trace::getCodonSpecificCategory(unsigned mixtureElement, unsigned param
 	case 1:
 		rv = categories->at(mixtureElement).delEta;
 		break;
+    case 2:
+        rv = categories->at(mixtureElement).delM;
+        break;
 	default:
 		my_printError("ERROR: Unknown parameter type in getCodonSpecificCategory\n");
 		break;
@@ -468,6 +515,39 @@ unsigned Trace::getCodonSpecificCategory(unsigned mixtureElement, unsigned param
 }
 
 
+//--------------------------------------//
+//----------- PANSE Specific -----------//
+//--------------------------------------//
+std::vector<double> Trace::getPartitionFunctionTrace(unsigned mixtureIndex)
+{
+    return partitionFunctionTrace[mixtureIndex];
+}
+
+std::vector<std::vector<double>> Trace::getPartitionFunctionTraces()
+{
+    return partitionFunctionTrace;
+}
+
+
+std::vector<double> Trace::getPartitionFunctionAcceptanceRateTrace()
+{
+    return partitionFunctionTraceAcceptanceRateTrace;
+}
+
+
+//--------------------------------------//
+//----------- FONSE Specific -----------//
+//--------------------------------------//
+std::vector<double> Trace::getInitiationCostTrace()
+{
+    return initiationCostTrace;
+}
+
+
+std::vector<double> Trace::getInitiationCostAcceptanceRateTrace()
+{
+    return initiationCostAcceptanceRateTrace;
+}
 
 
 
@@ -500,6 +580,7 @@ void Trace::updateCodonSpecificAcceptanceRateTrace(unsigned codonIndex, double a
 }
 
 
+
 void Trace::updateSynthesisRateTrace(unsigned sample, unsigned geneIndex,
 	std::vector<std::vector <double>> &currentSynthesisRateLevel)
 {
@@ -539,34 +620,11 @@ void Trace::updateCodonSpecificParameterTraceForAA(unsigned sample, std::string
 	{
 		for (unsigned i = aaStart; i < aaEnd; i++)
 		{
+			//my_print("% % % %\n",category,i,aa,curParam[category][i]);
 			codonSpecificParameterTrace[paramType][category][i][sample] = curParam[category][i];
 		}
 	}
-	/*
-	switch (paramType) {
-	case 0:
-		for (unsigned category = 0; category < codonSpecificParameterTraceOne.size(); category++)
-		{
-			for (unsigned i = aaStart; i < aaEnd; i++)
-			{
-				codonSpecificParameterTraceOne[category][i][sample] = curParam[category][i];
-			}
-		}
-		break;
-	case 1:
-		for (unsigned category = 0; category < codonSpecificParameterTraceTwo.size(); category++)
-		{
-			for (unsigned i = aaStart; i < aaEnd; i++)
-			{
-				codonSpecificParameterTraceTwo[category][i][sample] = curParam[category][i];
-			}
-		}
-		break;
-	default:
-		my_printError("ERROR: Unknown parameter type in updateCodonSpecificParameterTraceForAA\n");
-		break;
-	}
-	*/
+
 }
 
 
@@ -589,7 +647,7 @@ void Trace::updateObservedSynthesisNoiseTrace(unsigned index, unsigned sample, d
 
 
 //-------------------------------------//
-//---------- PA Specific -------------//
+//-------- PA/PANSE Specific ----------//
 //-------------------------------------//
 
 void Trace::updateCodonSpecificParameterTraceForCodon(unsigned sample, std::string codon,
@@ -598,13 +656,50 @@ void Trace::updateCodonSpecificParameterTraceForCodon(unsigned sample, std::stri
 	unsigned i = SequenceSummary::codonToIndex(codon);
 	for (unsigned category = 0; category < codonSpecificParameterTrace[paramType].size(); category++)
 	{
-        if(std::isnan(curParam[category][i])){
-            my_printError("\n Trace::updateCodonSpecificParameterTraceForCodon: Current parameter set contains NaN. \n");
-        }
+//        if(std::isnan(curParam[category][i])){
+//            my_printError("\n Trace::updateCodonSpecificParameterTraceForCodon: Current parameter set contains NaN. \n");
+//        }
 		codonSpecificParameterTrace[paramType][category][i][sample] = curParam[category][i];
 	}
 }
 
+void Trace::updatePartitionFunctionTrace(unsigned index, unsigned sample, double value)
+{
+   partitionFunctionTrace[index][sample] = value;
+}
+
+
+void Trace:: updatePartitionFunctionAcceptanceRateTrace(double value)
+{
+    partitionFunctionTraceAcceptanceRateTrace.push_back(value);
+}
+
+void Trace::updateNseRateSpecificAcceptanceRateTrace(unsigned codonIndex, double acceptanceLevel)
+{
+	nseSpecificAcceptanceRateTrace[codonIndex].push_back(acceptanceLevel);
+}
+
+//------------------------------------//
+//----------- FONSE Specific ---------//
+//------------------------------------//
+
+void Trace::updateInitiationCostTrace(unsigned sample, double value)
+{
+   initiationCostTrace[sample] = value;
+}
+
+
+void Trace:: updateInitiationCostAcceptanceRateTrace(double value)
+{
+    initiationCostAcceptanceRateTrace.push_back(value);
+}
+
+
+
+
+
+
+
 
 // -----------------------------------------------------------------------------------------------------//
 // ---------------------------------------- R SECTION --------------------------------------------------//
@@ -689,6 +784,16 @@ unsigned Trace::getNumberOfMixtures()
 	return mixtureProbabilitiesTrace.size();
 }
 
+std::vector<double> Trace::getPartitionFunctionTraceR(unsigned mixtureIndex){
+    std::vector<double> RV;
+    bool check = checkIndex(mixtureIndex, 1, partitionFunctionTrace.size());
+    if (check)
+    {
+        RV = getPartitionFunctionTrace(mixtureIndex - 1);
+    }
+    return RV;
+}
+
 //--------------------------------------//
 //---------- Setter Functions ----------//
 //--------------------------------------//
@@ -733,6 +838,11 @@ void Trace::setCodonSpecificAcceptanceRateTrace(std::vector<std::vector<double>>
 	codonSpecificAcceptanceRateTrace = _cspAcceptanceRateTrace;
 }
 
+void Trace::setNseRateSpecificAcceptanceRateTrace(std::vector<std::vector<double>> _nseAcceptanceRateTrace)
+{
+	nseSpecificAcceptanceRateTrace = _nseAcceptanceRateTrace;
+}
+
 
 void Trace::setCategories(std::vector<mixtureDefinition> &_categories)
 {
@@ -803,6 +913,27 @@ void Trace::setCodonSpecificParameterTrace(std::vector<std::vector<std::vector<f
 }
 
 
+//----------------------------------//
+//--------- PANSE Specific ---------//
+//----------------------------------//
+void Trace::setPartitionFunctionTraces(std::vector<std::vector <double> > _PartitionFunctionTrace)
+{
+    partitionFunctionTrace = _PartitionFunctionTrace;
+}
+
+
+//----------------------------------//
+//--------- FONSE Specific ---------//
+//----------------------------------//
+void Trace::setInitiationCostTrace(std::vector <double> _InitiationCostTrace)
+{
+    initiationCostTrace = _InitiationCostTrace;
+}
+
+
+
+
+
 bool Trace::checkIndex(unsigned index, unsigned lowerbound, unsigned upperbound)
 {
 	bool check = false;
@@ -819,3 +950,11 @@ bool Trace::checkIndex(unsigned index, unsigned lowerbound, unsigned upperbound)
 }
 
 #endif
+
+
+//Function implemented to resize codonSpecificParameterTrace for reloading parameter object.
+//TO DO: Think of more elegant solution. 
+void Trace::resizeNumberCodonSpecificParameterTrace(unsigned _numCodonSpecificParamTypes)
+{
+	codonSpecificParameterTrace.resize(_numCodonSpecificParamTypes);
+}
