@@ -3,6 +3,7 @@
 #include "include/SequenceSummary.h"
 #include <vector>
 #include <iostream>
+#include <fstream>
 #include <cstdio>
 #include <cstdlib>
 
@@ -1324,3 +1325,101 @@ int main()
 }
 
 #endif // Denizhan
+
+#ifdef ALEX
+int main()
+{
+	srand(1500);
+    std::vector <double> alphas;
+    std::vector <double> lambdas;
+    std::vector <std::string> cspFiles;
+    std::string pathBegin = "/home/acope3/Panse_project/Logs/Input";
+    Genome genome;
+	unsigned numMixtures = 1;
+	std::vector<double> sphi_init(numMixtures, 1);
+	std::vector<unsigned> geneAssignment;
+	genome.readRFPData("/home/acope3/Panse_project/Logs/Input/Stochastic_Simulation/Alex_simulations_10_24_remove_stops.csv", false);
+    geneAssignment.resize(genome.getGenomeSize());
+    my_print("%\n", genome.getGenomeSize());
+	for (unsigned i = 0u; i < genome.getGenomeSize(); i++)
+	{
+		geneAssignment[i] = 0u;
+	}
+
+	std::vector<double> phi;
+	std::size_t pos;
+	std::ifstream currentFile;
+	std::string tmpString;
+	my_print("Initializing gene expression...\n");
+	currentFile.open("/home/acope3/Panse_project/Input/PopData/orderedRandGeneIDPhiMean.csv");
+	currentFile >> tmpString;
+	while (currentFile >> tmpString)
+	{
+		pos = tmpString.find(',');
+		if (pos != std::string::npos)
+		{
+			std::string val = tmpString.substr(pos + 1, std::string::npos);
+			phi.push_back(std::atof(val.c_str()));
+		}
+	}
+	my_print("Initializing CSP\n");
+	std::vector<std::vector<unsigned>> mixtureDefinitionMatrix;
+	std::string mixDef = PANSEParameter::allUnique;
+	PANSEParameter parameter(sphi_init, numMixtures, geneAssignment, mixtureDefinitionMatrix, true, mixDef);
+    cspFiles.push_back("/home/acope3/Panse_project/Input/PopData/JeremyRFPAlphaValues.csv");
+    parameter.initMutationSelectionCategories(cspFiles, 1, parameter.alp);
+    cspFiles[0] = ("/home/acope3/Panse_project/Input/PopData/JeremyRFPLambdaPrimeValues.csv");
+    parameter.initMutationSelectionCategories(cspFiles, 1, parameter.lmPri);
+    cspFiles[0] = ("/home/acope3/Panse_project/Logs/Input/Stochastic_Simulation/simNSEMay_4.csv");
+    parameter.initMutationSelectionCategories(cspFiles, 1, parameter.nse);
+    parameter.InitializeSynthesisRate(phi);
+	PANSEModel model;
+	model.setParameter(parameter);
+
+	my_print("Initializing MCMCAlgorithm object---------------\n");
+	unsigned samples = 50;
+	unsigned thinning = 2;
+
+	my_print("\t# Samples: %\n", samples);
+	my_print("\tThinning: %\n", thinning);
+	MCMCAlgorithm mcmc = MCMCAlgorithm(samples, thinning, 10, false, true, false);
+	mcmc.setRestartFileSettings("RestartFile.txt", 20, true);
+	my_print("Done!-------------------------------\n\n\n");
+
+	my_print("Done!--------------------------------\n\n\n");
+
+
+	my_print("Initializing PAModel object--------------------------\n");
+	model.setParameter(parameter);
+	my_print("Done!----------------------------------\n\n\n");
+
+
+	my_print("Running MCMC.............\n\n");
+	mcmc.run(genome, model, 1, 0);
+	my_print("Done!----------------------------------\n\n\n");
+
+	std::vector<std::string> codons = parameter.getGroupList();
+	std::vector<double> alpha,lmprime,nse;
+	double tmp;
+	for (unsigned i=0; i < codons.size();i++)
+	{
+		tmp = parameter.getCodonSpecificPosteriorMean(0, samples, codons[i],0, true, false);
+		alpha.push_back(tmp);
+		tmp = parameter.getCodonSpecificPosteriorMean(0, samples, codons[i],1, true, false);
+		lmprime.push_back(tmp);
+		tmp = parameter.getCodonSpecificPosteriorMean(0, samples, codons[i],2, true, false);
+		nse.push_back(tmp);
+	}
+	std::ofstream myFile("cpp_runs_10_24.csv");
+	myFile << "Codon,Alpha,LambdaPrime,NSE\n";
+	for (unsigned i=0;i < codons.size();i++)
+	{
+		myFile << codons[i] <<","<<alpha[i]<<","<<lmprime[i]<<","<<nse[i]<<"\n";
+	}
+	std::cout << mcmc.getLogPosteriorMean(samples);
+
+
+}
+
+#endif //Alex
+
